@@ -2,16 +2,29 @@
 
 import calendar
 import json
-import platform
+import os
+import unittest
 import warnings
 import yaml
 from contextlib import redirect_stdout
 from enum import Enum
 from io import StringIO
 from typing import Any, Dict, List, Optional, Tuple, Union
-from jsonargparse_tests.base import *
+from jsonargparse import (
+    ActionConfigFile,
+    ArgumentParser,
+    class_from_function,
+    compose_dataclasses,
+    lazy_instance,
+    Namespace,
+    ParserError,
+    strip_meta,
+)
 from jsonargparse.actions import _find_action
+from jsonargparse.optionals import dataclasses_support, docstring_parser_support, import_dataclasses
+from jsonargparse.typing import final, OpenUnitInterval, PositiveFloat, PositiveInt
 from jsonargparse.util import _suppress_stderr
+from jsonargparse_tests.base import mock_module, TempDirTestCase
 
 
 class SignaturesTests(unittest.TestCase):
@@ -155,7 +168,6 @@ class SignaturesTests(unittest.TestCase):
         parser = ArgumentParser(error_handler=None)
         parser.add_function_arguments(func)
         parser.get_defaults()
-        cfg = parser.parse_args(['--a1={"2": 7, "4": 9}'])
         self.assertEqual({2: 7, 4: 9}, parser.parse_args(['--a1={"2": 7, "4": 9}']).a1)
 
 
@@ -295,36 +307,37 @@ class SignaturesTests(unittest.TestCase):
             parser.parse_args(['--cal.help=calendar.Calendar'])
         self.assertIn('--cal.init_args.firstweekday', out.getvalue())
 
-        if True:  # platform.python_implementation() == 'CPython':
-            class MyCalendar(calendar.Calendar):
-                init_called = False
-                getfirst = calendar.Calendar.getfirstweekday
-                def __init__(self, *args, **kwargs):
-                    self.init_called = True
-                    super().__init__(*args, **kwargs)
+        # lazy_instance
+        class MyCalendar(calendar.Calendar):
+            init_called = False
+            getfirst = calendar.Calendar.getfirstweekday
+            def __init__(self, *args, **kwargs):
+                self.init_called = True
+                super().__init__(*args, **kwargs)
 
-            lazy_calendar = lazy_instance(MyCalendar, firstweekday=3)
-            self.assertFalse(lazy_calendar.init_called, '__init__ was already called but supposed to be lazy')
-            self.assertEqual(lazy_calendar.getfirstweekday(), 3)
-            self.assertTrue(lazy_calendar.init_called)
+        lazy_calendar = lazy_instance(MyCalendar, firstweekday=3)
+        self.assertFalse(lazy_calendar.init_called, '__init__ was already called but supposed to be lazy')
+        self.assertEqual(lazy_calendar.getfirstweekday(), 3)
+        self.assertTrue(lazy_calendar.init_called)
 
-            cal['init_args']['firstweekday'] = 4
-            lazy_calendar = lazy_instance(calendar.Calendar, firstweekday=4)
-            parser.set_defaults({'cal': lazy_calendar})
-            cfg = parser.parse_string(parser.dump(parser.parse_args([])))
-            self.assertEqual(cfg['cal'].as_dict(), cal)
-            self.assertEqual(lazy_calendar.getfirstweekday(), 4)
+        cal['init_args']['firstweekday'] = 4
+        lazy_calendar = lazy_instance(calendar.Calendar, firstweekday=4)
+        parser.set_defaults({'cal': lazy_calendar})
+        cfg = parser.parse_string(parser.dump(parser.parse_args([])))
+        self.assertEqual(cfg['cal'].as_dict(), cal)
+        self.assertEqual(lazy_calendar.getfirstweekday(), 4)
 
-            parser.add_argument('--config', action=ActionConfigFile)
-            out = StringIO()
-            with redirect_stdout(out), self.assertRaises(SystemExit):
-                parser.parse_args(['--print_config'])
-            self.assertIn('class_path: calendar.Calendar', out.getvalue())
+        parser.add_argument('--config', action=ActionConfigFile)
+        out = StringIO()
+        with redirect_stdout(out), self.assertRaises(SystemExit):
+            parser.parse_args(['--print_config'])
+        self.assertIn('class_path: calendar.Calendar', out.getvalue())
 
-            out = StringIO()
-            parser.print_help(out)
-            self.assertIn("'init_args': {'firstweekday': 4}", out.getvalue())
+        out = StringIO()
+        parser.print_help(out)
+        self.assertIn("'init_args': {'firstweekday': 4}", out.getvalue())
 
+        # defaults
         parser.set_defaults({'cal': calendar.Calendar(firstweekday=4)})
         cfg = parser.parse_args([])
         self.assertIsInstance(cfg['cal'], calendar.Calendar)
@@ -340,22 +353,21 @@ class SignaturesTests(unittest.TestCase):
                 self.param = param
                 super().__init__(*args, **kwargs)
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'MyCal', MyCal)
-        args = ['--cal.help=jsonargparse_tests.signatures_tests.MyCal']
+        with mock_module(MyCal) as module:
+            args = [f'--cal.help={module}.MyCal']
 
-        parser = ArgumentParser()
-        parser.add_subclass_arguments(calendar.Calendar, 'cal', skip={'param'})
+            parser = ArgumentParser()
+            parser.add_subclass_arguments(calendar.Calendar, 'cal', skip={'param'})
 
-        out = StringIO()
-        with redirect_stdout(out), self.assertRaises(SystemExit):
-            parser.parse_args(args)
-        self.assertIn('--cal.init_args.firstweekday', out.getvalue())
-        self.assertNotIn('param', out.getvalue())
+            out = StringIO()
+            with redirect_stdout(out), self.assertRaises(SystemExit):
+                parser.parse_args(args)
+            self.assertIn('--cal.init_args.firstweekday', out.getvalue())
+            self.assertNotIn('param', out.getvalue())
 
-        parser = ArgumentParser(error_handler=None)
-        parser.add_subclass_arguments(calendar.Calendar, 'cal')
-        self.assertRaises(ValueError, lambda: parser.parse_args(args))
+            parser = ArgumentParser(error_handler=None)
+            parser.add_subclass_arguments(calendar.Calendar, 'cal')
+            self.assertRaises(ValueError, lambda: parser.parse_args(args))
 
 
     def test_add_subclass_discard_init_args(self):
@@ -365,15 +377,12 @@ class SignaturesTests(unittest.TestCase):
         class MyCal(calendar.Calendar):
             pass
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'MyCal', MyCal)
-
-        with warnings.catch_warnings(record=True) as w:
+        with mock_module(MyCal) as module, warnings.catch_warnings(record=True) as w:
             warnings.simplefilter("always")
             parser.parse_args([
                 '--cal.class_path=calendar.Calendar',
                 '--cal.init_args.firstweekday=4',
-                '--cal.class_path=jsonargparse_tests.signatures_tests.MyCal',
+                f'--cal.class_path={module}.MyCal',
             ])
             self.assertEqual(len(w), 1)
             self.assertIn("discarding init_args {'firstweekday': 4} defined for class_path calendar.Calendar", str(w[0].message))
@@ -414,30 +423,28 @@ class SignaturesTests(unittest.TestCase):
                 self.b1 = b1
                 self.b2 = b2
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'ClassA', ClassA)
-        setattr(signatures_tests, 'ClassB', ClassB)
-        class_path_a = 'jsonargparse_tests.signatures_tests.ClassA'
-        class_path_b = 'jsonargparse_tests.signatures_tests.ClassB'
+        with mock_module(ClassA, ClassB) as module:
+            class_path_a = f'{module}.ClassA'
+            class_path_b = f'{module}.ClassB'
 
-        parser = ArgumentParser(error_handler=None)
-        parser.add_subclass_arguments((ClassA, ClassB), 'c')
+            parser = ArgumentParser(error_handler=None)
+            parser.add_subclass_arguments((ClassA, ClassB), 'c')
 
-        cfg = parser.parse_args(['--c={"class_path": "'+class_path_a+'", "init_args": {"a1": -1}}'])
-        self.assertEqual(cfg.c.init_args.a1, -1)
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['c'], ClassA)
+            cfg = parser.parse_args(['--c={"class_path": "'+class_path_a+'", "init_args": {"a1": -1}}'])
+            self.assertEqual(cfg.c.init_args.a1, -1)
+            cfg = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg['c'], ClassA)
 
-        cfg = parser.parse_args(['--c={"class_path": "'+class_path_b+'", "init_args": {"b1": -4.5}}'])
-        self.assertEqual(cfg.c.init_args.b1, -4.5)
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['c'], ClassB)
+            cfg = parser.parse_args(['--c={"class_path": "'+class_path_b+'", "init_args": {"b1": -4.5}}'])
+            self.assertEqual(cfg.c.init_args.b1, -4.5)
+            cfg = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg['c'], ClassB)
 
-        out = StringIO()
-        with redirect_stdout(out), self.assertRaises(SystemExit):
-            parser.parse_args(['--c.help='+class_path_b])
+            out = StringIO()
+            with redirect_stdout(out), self.assertRaises(SystemExit):
+                parser.parse_args(['--c.help='+class_path_b])
 
-        self.assertIn('--c.init_args.b1', out.getvalue())
+            self.assertIn('--c.init_args.b1', out.getvalue())
 
 
     def test_required_group(self):
@@ -544,12 +551,9 @@ class SignaturesTests(unittest.TestCase):
         parser = ArgumentParser(error_handler=None)
         parser.add_class_arguments(Class2, skip={'c1.init_args.a2', 'c2'})
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'Class1', Class1)
-        class_path = 'jsonargparse_tests.signatures_tests.Class1'
-
-        cfg = parser.parse_args(['--c1='+class_path])
-        self.assertEqual(cfg.c1.init_args, Namespace(a1=1, a3='4'))
+        with mock_module(Class1) as module:
+            cfg = parser.parse_args([f'--c1={module}.Class1'])
+            self.assertEqual(cfg.c1.init_args, Namespace(a1=1, a3='4'))
 
 
     def test_skip_in_add_subclass_arguments(self):
@@ -568,16 +572,14 @@ class SignaturesTests(unittest.TestCase):
         parser = ArgumentParser(error_handler=None)
         parser.add_subclass_arguments(ClassA, 'c', skip={'a1', 'b2'})
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'ClassA', ClassA)
-        setattr(signatures_tests, 'ClassB', ClassB)
-        class_path_a = 'jsonargparse_tests.signatures_tests.ClassA'
-        class_path_b = 'jsonargparse_tests.signatures_tests.ClassB'
+        with mock_module(ClassA, ClassB) as module:
+            class_path_a = f'{module}.ClassA'
+            class_path_b = f'{module}.ClassB'
 
-        cfg = parser.parse_args(['--c='+class_path_a])
-        self.assertEqual(cfg.c.init_args, Namespace(a2=2.3))
-        cfg = parser.parse_args(['--c='+class_path_b])
-        self.assertEqual(cfg.c.init_args, Namespace(a2=2.3, b1=4.5))
+            cfg = parser.parse_args(['--c='+class_path_a])
+            self.assertEqual(cfg.c.init_args, Namespace(a2=2.3))
+            cfg = parser.parse_args(['--c='+class_path_b])
+            self.assertEqual(cfg.c.init_args, Namespace(a2=2.3, b1=4.5))
 
 
     def test_final_class(self):
@@ -703,35 +705,33 @@ class SignaturesTests(unittest.TestCase):
         parser = ArgumentParser(error_handler=None)
         parser.add_class_arguments(Class2)
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'Class1', Class1)
+        with mock_module(Class1) as module:
+            class_path = f'"class_path": "{module}.Class1"'
+            init_args = '"init_args": {"a1": 7}'
+            cfg = parser.parse_args(['--c1={'+class_path+', '+init_args+'}'])
+            self.assertEqual(cfg.c1.class_path, f'{module}.Class1')
+            self.assertEqual(cfg.c1.init_args, Namespace(a1=7, a2=2.3))
+            cfg = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg['c1'], Class1)
+            self.assertEqual(7, cfg['c1'].a1)
+            self.assertEqual(2.3, cfg['c1'].a2)
 
-        class_path = '"class_path": "jsonargparse_tests.signatures_tests.Class1"'
-        init_args = '"init_args": {"a1": 7}'
-        cfg = parser.parse_args(['--c1={'+class_path+', '+init_args+'}'])
-        self.assertEqual(cfg.c1.class_path, 'jsonargparse_tests.signatures_tests.Class1')
-        self.assertEqual(cfg.c1.init_args, Namespace(a1=7, a2=2.3))
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['c1'], Class1)
-        self.assertEqual(7, cfg['c1'].a1)
-        self.assertEqual(2.3, cfg['c1'].a2)
+            parser = ArgumentParser(error_handler=None)
+            parser.add_class_arguments(Class2, 'c2')
 
-        parser = ArgumentParser(error_handler=None)
-        parser.add_class_arguments(Class2, 'c2')
+            cfg = parser.parse_args(['--c2={"c1": {'+class_path+', '+init_args+'}}'])
+            cfg = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg['c2'], Class2)
+            self.assertIsInstance(cfg['c2'].c1, Class1)
 
-        cfg = parser.parse_args(['--c2={"c1": {'+class_path+', '+init_args+'}}'])
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['c2'], Class2)
-        self.assertIsInstance(cfg['c2'].c1, Class1)
+            class EmptyInitClass:
+                pass
 
-        class EmptyInitClass:
-            pass
-
-        parser = ArgumentParser(error_handler=None)
-        parser.add_class_arguments(EmptyInitClass, 'e')
-        cfg = parser.parse_args([])
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['e'], EmptyInitClass)
+            parser = ArgumentParser(error_handler=None)
+            parser.add_class_arguments(EmptyInitClass, 'e')
+            cfg = parser.parse_args([])
+            cfg = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg['e'], EmptyInitClass)
 
 
     def test_instantiate_classes_subcommand(self):
@@ -786,7 +786,7 @@ class SignaturesTests(unittest.TestCase):
                 """
                 pass
 
-        with mock.patch('docstring_parser.parse') as docstring_parse:
+        with unittest.mock.patch('docstring_parser.parse') as docstring_parse:
             docstring_parse.side_effect = ValueError
             parser = ArgumentParser(error_handler=None)
             parser.add_class_arguments(Class1)
@@ -818,6 +818,33 @@ class SignaturesTests(unittest.TestCase):
         self.assertEqual(outval['g'], {'a1': {'class_path': 'calendar.Calendar', 'init_args': {'firstweekday': 0}}, 'a2': 7})
 
 
+    def test_print_config_subclass_required_param_issue_115(self):
+        class Class(object):
+            def __init__(self, arg1: float):
+                pass
+
+        class BaseClass(object):
+            def __init__(self):
+                pass
+
+        class SubClass(BaseClass):
+            def __init__(self, arg1: int, arg2: int = 1):
+                pass
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument("--config", action=ActionConfigFile)
+        parser.add_class_arguments(Class, 'class')
+        parser.add_subclass_arguments(BaseClass, 'subclass')
+
+        with mock_module(BaseClass, SubClass) as module:
+            args = [f'--subclass={module}.SubClass', '--print_config']
+            out = StringIO()
+            with redirect_stdout(out), self.assertRaises(SystemExit):
+                parser.parse_args(args)
+            expected = f'class:\n  arg1: null\nsubclass:\n  class_path: {module}.SubClass\n  init_args:\n    arg1: null\n    arg2: 1\n'
+            self.assertEqual(out.getvalue(), expected)
+
+
     def test_link_arguments(self):
 
         class ClassA:
@@ -875,9 +902,6 @@ class SignaturesTests(unittest.TestCase):
             ):
                 pass
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'ClassA', ClassA)
-
         parser = ArgumentParser(error_handler=None)
         parser.add_subclass_arguments(ClassA, 'a')
         parser.add_subclass_arguments(calendar.Calendar, 'c')
@@ -886,28 +910,29 @@ class SignaturesTests(unittest.TestCase):
             return v1 + v2
         parser.link_arguments(('a.init_args.v1', 'a.init_args.v2'), 'c.init_args.firstweekday', add)
 
-        a_value = {
-            'class_path': 'jsonargparse_tests.signatures_tests.ClassA',
-            'init_args': {'v2': 3},
-        }
+        with mock_module(ClassA) as module:
+            a_value = {
+                'class_path': f'{module}.ClassA',
+                'init_args': {'v2': 3},
+            }
 
-        cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
-        self.assertEqual(cfg.c.init_args.firstweekday, 4)
-        self.assertEqual(cfg.c.init_args.firstweekday, cfg.a.init_args.v1+cfg.a.init_args.v2)
+            cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
+            self.assertEqual(cfg.c.init_args.firstweekday, 4)
+            self.assertEqual(cfg.c.init_args.firstweekday, cfg.a.init_args.v1+cfg.a.init_args.v2)
 
-        cfg_init = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg_init.a, ClassA)
-        self.assertIsInstance(cfg_init.c, calendar.Calendar)
+            cfg_init = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg_init.a, ClassA)
+            self.assertIsInstance(cfg_init.c, calendar.Calendar)
 
-        dump = yaml.safe_load(parser.dump(cfg))
-        a_value['init_args']['v1'] = 1
-        self.assertEqual(dump, {'a': a_value, 'c': {'class_path': 'calendar.Calendar'}})
+            dump = yaml.safe_load(parser.dump(cfg))
+            a_value['init_args']['v1'] = 1
+            self.assertEqual(dump, {'a': a_value, 'c': {'class_path': 'calendar.Calendar'}})
 
-        self.assertRaises(ValueError, lambda: parser.link_arguments('a.init_args.v1', 'c'))
+            self.assertRaises(ValueError, lambda: parser.link_arguments('a.init_args.v1', 'c'))
 
-        a_value['init_args'] = {'v1': 'a', 'v2': 'b'}
-        with self.assertRaises(ParserError):
-            parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
+            a_value['init_args'] = {'v1': 'a', 'v2': 'b'}
+            with self.assertRaises(ParserError):
+                parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
 
 
     def test_link_arguments_subclasses_with_instantiate_false(self):
@@ -919,34 +944,32 @@ class SignaturesTests(unittest.TestCase):
             ):
                 self.c = c
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'ClassA', ClassA)
-
         parser = ArgumentParser(error_handler=None)
         parser.add_subclass_arguments(ClassA, 'a')
         parser.add_subclass_arguments(calendar.Calendar, 'c', instantiate=False)
         parser.link_arguments('c', 'a.init_args.c')
 
-        a_value = {'class_path': 'jsonargparse_tests.signatures_tests.ClassA'}
-        c_value = {
-            'class_path': 'calendar.Calendar',
-            'init_args': {
-                'firstweekday': 3,
-            },
-        }
+        with mock_module(ClassA) as module:
+            a_value = {'class_path': f'{module}.ClassA'}
+            c_value = {
+                'class_path': 'calendar.Calendar',
+                'init_args': {
+                    'firstweekday': 3,
+                },
+            }
 
-        cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c='+json.dumps(c_value)])
-        self.assertEqual(cfg.c.as_dict(), {'class_path': 'calendar.Calendar', 'init_args': {'firstweekday': 3}})
-        self.assertEqual(cfg.c, cfg.a.init_args.c)
+            cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c='+json.dumps(c_value)])
+            self.assertEqual(cfg.c.as_dict(), {'class_path': 'calendar.Calendar', 'init_args': {'firstweekday': 3}})
+            self.assertEqual(cfg.c, cfg.a.init_args.c)
 
-        cfg_init = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg_init.c, Namespace)
-        self.assertIsInstance(cfg_init.a, ClassA)
-        self.assertIsInstance(cfg_init.a.c, calendar.Calendar)
-        self.assertEqual(cfg_init.a.c.firstweekday, 3)
+            cfg_init = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg_init.c, Namespace)
+            self.assertIsInstance(cfg_init.a, ClassA)
+            self.assertIsInstance(cfg_init.a.c, calendar.Calendar)
+            self.assertEqual(cfg_init.a.c.firstweekday, 3)
 
-        dump = yaml.safe_load(parser.dump(cfg))
-        self.assertNotIn('c', dump['a']['init_args'])
+            dump = yaml.safe_load(parser.dump(cfg))
+            self.assertNotIn('c', dump['a']['init_args'])
 
 
     def test_link_arguments_subclass_as_dict(self):
@@ -962,9 +985,6 @@ class SignaturesTests(unittest.TestCase):
         def return_dict(value: dict):
             return value
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'ClassA', ClassA)
-
         parser = ArgumentParser(error_handler=None)
         parser.add_subclass_arguments(ClassA, 'a')
         parser.add_subclass_arguments(calendar.Calendar, 'c')
@@ -972,17 +992,18 @@ class SignaturesTests(unittest.TestCase):
         parser.link_arguments('c', 'a.init_args.a2')
         parser.link_arguments('c', 'a.init_args.a3')
 
-        a_value = {'class_path': 'jsonargparse_tests.signatures_tests.ClassA'}
-        c_value = {
-            'class_path': 'calendar.Calendar',
-            'init_args': {
-                'firstweekday': 3,
-            },
-        }
+        with mock_module(ClassA) as module:
+            a_value = {'class_path': f'{module}.ClassA'}
+            c_value = {
+                'class_path': 'calendar.Calendar',
+                'init_args': {
+                    'firstweekday': 3,
+                },
+            }
 
-        cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c='+json.dumps(c_value)])
-        self.assertEqual(cfg.a.init_args.a1, c_value)
-        self.assertEqual(cfg.a.init_args.a2, c_value)
+            cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c='+json.dumps(c_value)])
+            self.assertEqual(cfg.a.init_args.a1, c_value)
+            self.assertEqual(cfg.a.init_args.a2, c_value)
 
 
     def test_link_arguments_subcommand(self):
@@ -1024,10 +1045,6 @@ class SignaturesTests(unittest.TestCase):
                 self.c1 = c1
                 self.c2 = c2
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'ClassA', ClassA)
-        setattr(signatures_tests, 'ClassB', ClassB)
-
         def make_parser_1():
             parser = ArgumentParser(error_handler=None)
             parser.add_class_arguments(ClassA, 'a')
@@ -1078,13 +1095,14 @@ class SignaturesTests(unittest.TestCase):
             parser.link_arguments('d', 'c.c2', apply_on='instantiate')
 
         # Link subclass and compute function
-        parser = make_parser_2()
-        cfg = parser.parse_args([
-            '--a=jsonargparse_tests.signatures_tests.ClassA',
-            '--b=jsonargparse_tests.signatures_tests.ClassB',
-        ])
-        cfg = parser.instantiate_classes(cfg)
-        self.assertEqual(cfg['a'].a1, 6)
+        with mock_module(ClassA, ClassB) as module:
+            parser = make_parser_2()
+            cfg = parser.parse_args([
+                f'--a={module}.ClassA',
+                f'--b={module}.ClassB',
+            ])
+            cfg = parser.instantiate_classes(cfg)
+            self.assertEqual(cfg['a'].a1, 6)
 
         # Unsupported multi-source
         parser = make_parser_2()
@@ -1109,6 +1127,40 @@ class SignaturesTests(unittest.TestCase):
             parser.link_arguments('g.b.b2', 'g.a.a1', apply_on='instantiate')
 
 
+    def test_class_from_function(self):
+
+        def get_calendar() -> calendar.Calendar:
+            return calendar.Calendar()
+
+        class Foo:
+            @classmethod
+            def get_foo(cls) -> 'Foo':
+                return cls()
+
+        def closure_get_foo():
+            def get_foo() -> 'Foo':
+                return Foo()
+            return get_foo
+
+        for function, class_type in [
+            (get_calendar, calendar.Calendar),
+            (Foo.get_foo, Foo),
+            (closure_get_foo(), Foo),
+        ]:
+            with self.subTest(str((function, class_type))):
+                cls = class_from_function(function)
+                self.assertTrue(issubclass(cls, class_type))
+                self.assertIsInstance(cls(), class_type)
+
+
+    def test_invalid_class_from_function(self):
+
+        def get_unknown() -> 'Unknown':  # type: ignore
+            return None
+
+        self.assertRaises(ValueError, lambda: class_from_function(get_unknown))
+
+
     def test_add_class_from_function_arguments(self):
 
         def get_calendar(a1: str, a2: int = 2) -> calendar.Calendar:
@@ -1149,7 +1201,7 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, encoder: Module):
                 pass
 
-        with mock_module(Module=Module, Network=Network, Model=Model) as module:
+        with mock_module(Module, Network, Model) as module:
 
             config = f"""model:
               encoder:
@@ -1295,7 +1347,7 @@ class SignaturesConfigTests(TempDirTestCase):
             def __init__(self, sub_module: SubModule):
                 pass
 
-        with mock_module(SubModule=SubModule, Model=Model) as module:
+        with mock_module(SubModule, Model) as module:
 
             defaults = f"""model:
               sub_module:
@@ -1432,20 +1484,18 @@ class DataclassesTests(unittest.TestCase):
         parser = ArgumentParser(error_handler=None)
         parser.add_class_arguments(MyClass2)
 
-        from jsonargparse_tests import signatures_tests
-        setattr(signatures_tests, 'MyClass1', MyClass1)
-
-        class_path = '"class_path": "jsonargparse_tests.signatures_tests.MyClass1"'
-        init_args = '"init_args": {"a1": {"b2": {"a1": 7}}}'
-        cfg = parser.parse_args(['--c1={'+class_path+', '+init_args+'}'])
-        self.assertEqual(cfg.c1.class_path, 'jsonargparse_tests.signatures_tests.MyClass1')
-        self.assertEqual(cfg.c1.init_args.a1.b2.a1, 7)
-        self.assertIsInstance(cfg.c1.init_args.a1.b2.a1, PositiveInt)
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['c1'], MyClass1)
-        self.assertIsInstance(cfg['c1'].a1, self.MyDataClassB)
-        self.assertIsInstance(cfg['c1'].a1.b2, self.MyDataClassA)
-        self.assertIsInstance(cfg['c1'].a1.b1, PositiveFloat)
+        with mock_module(MyClass1, MyClass2) as module:
+            class_path = f'"class_path": "{module}.MyClass1"'
+            init_args = '"init_args": {"a1": {"b2": {"a1": 7}}}'
+            cfg = parser.parse_args(['--c1={'+class_path+', '+init_args+'}'])
+            self.assertEqual(cfg.c1.class_path, f'{module}.MyClass1')
+            self.assertEqual(cfg.c1.init_args.a1.b2.a1, 7)
+            self.assertIsInstance(cfg.c1.init_args.a1.b2.a1, PositiveInt)
+            cfg = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg['c1'], MyClass1)
+            self.assertIsInstance(cfg['c1'].a1, self.MyDataClassB)
+            self.assertIsInstance(cfg['c1'].a1.b2, self.MyDataClassA)
+            self.assertIsInstance(cfg['c1'].a1.b1, PositiveFloat)
 
 
     def test_dataclass_add_argument_type(self):
