@@ -1,16 +1,30 @@
 #!/usr/bin/env python3
 
 import json
+import os
 import pathlib
 import sys
+import unittest
 import uuid
 import yaml
 from calendar import Calendar
 from datetime import datetime
 from enum import Enum
 from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Sequence, Tuple, Type, Union
-from jsonargparse_tests.base import *
-from jsonargparse.typehints import is_optional, Literal
+from jsonargparse import ActionConfigFile, ArgumentParser, CLI, lazy_instance, Namespace, ParserError, Path
+from jsonargparse.typehints import ActionTypeHint, is_optional, Literal
+from jsonargparse.typing import (
+    Email,
+    NotEmptyStr,
+    OpenUnitInterval,
+    Path_drw,
+    Path_fc,
+    Path_fr,
+    path_type,
+    PositiveInt,
+    register_type,
+)
+from jsonargparse_tests.base import mock_module, TempDirTestCase
 
 
 class TypeHintsTests(unittest.TestCase):
@@ -324,11 +338,9 @@ class TypeHintsTests(unittest.TestCase):
         parser = ArgumentParser(error_handler=None)
         parser.add_argument('--op', type=MyCal)
 
-        import jsonargparse_tests
-        setattr(jsonargparse_tests, 'MyCal', MyCal)
-
-        cfg = parser.parse_args(['--op.class_path=jsonargparse_tests.MyCal', '--op.init_args.p1=3'], defaults=False)
-        self.assertEqual(cfg.op, Namespace(class_path='jsonargparse_tests.MyCal', init_args=Namespace(p1=3)))
+        with mock_module(MyCal) as module:
+            cfg = parser.parse_args([f'--op.class_path={module}.MyCal', '--op.init_args.p1=3'], defaults=False)
+            self.assertEqual(cfg.op, Namespace(class_path=f'{module}.MyCal', init_args=Namespace(p1=3)))
 
 
     def test_class_type_required_params(self):
@@ -336,7 +348,7 @@ class TypeHintsTests(unittest.TestCase):
             def __init__(self, p1: int, p2: str):
                 pass
 
-        with mock_module(MyCal=MyCal) as module:
+        with mock_module(MyCal) as module:
             parser = ArgumentParser(error_handler=None)
             parser.add_argument('--op', type=MyCal, default=lazy_instance(MyCal))
 
@@ -535,25 +547,23 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             def __init__(self, *args, param: str = '0', **kwargs):
                 super().__init__(*args, **kwargs)
 
-        import jsonargparse_tests
-        setattr(jsonargparse_tests, 'MyCalendar', MyCalendar)
+        with mock_module(MyCalendar) as module:
+            config = {
+                'class_path': f'{module}.MyCalendar',
+                'init_args': {'firstweekday': 2, 'param': '1'},
+            }
+            config_path = os.path.join(self.tmpdir, 'config.yaml')
+            with open(config_path, 'w') as f:
+                json.dump({'cal': config}, f)
 
-        config = {
-            'class_path': 'jsonargparse_tests.MyCalendar',
-            'init_args': {'firstweekday': 2, 'param': '1'},
-        }
-        config_path = os.path.join(self.tmpdir, 'config.yaml')
-        with open(config_path, 'w') as f:
-            json.dump({'cal': config}, f)
+            parser = ArgumentParser(error_handler=None, default_config_files=[config_path])
+            parser.add_argument('--cal', type=Optional[Calendar])
 
-        parser = ArgumentParser(error_handler=None, default_config_files=[config_path])
-        parser.add_argument('--cal', type=Optional[Calendar])
+            cfg = parser.instantiate_classes(parser.get_defaults())
+            self.assertIsInstance(cfg['cal'], MyCalendar)
 
-        cfg = parser.instantiate_classes(parser.get_defaults())
-        self.assertIsInstance(cfg['cal'], MyCalendar)
-
-        cfg = parser.parse_args(['--cal={"class_path": "calendar.Calendar", "init_args": {"firstweekday": 3}}'])
-        self.assertEqual(type(parser.instantiate_classes(cfg)['cal']), Calendar)
+            cfg = parser.parse_args(['--cal={"class_path": "calendar.Calendar", "init_args": {"firstweekday": 3}}'])
+            self.assertEqual(type(parser.instantiate_classes(cfg)['cal']), Calendar)
 
 
     def test_linking_deep_targets(self):
@@ -575,37 +585,31 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             def fn(self) -> D:
                 return D()
 
-        config = {
-            "b": {
-                "class_path": "jsonargparse_tests.BSub",
-                "init_args": {
-                    "a": {
-                        "class_path": "jsonargparse_tests.A",
+        with mock_module(D, A, BSuper, BSub, C) as module:
+            config = {
+                "b": {
+                    "class_path": f"{module}.BSub",
+                    "init_args": {
+                        "a": {
+                            "class_path": f"{module}.A",
+                        },
                     },
                 },
-            },
-            "c": {},
-        }
-        config_path = os.path.join(self.tmpdir, 'config.yaml')
-        with open(config_path, 'w') as f:
-            yaml.safe_dump(config, f)
+                "c": {},
+            }
+            config_path = os.path.join(self.tmpdir, 'config.yaml')
+            with open(config_path, 'w') as f:
+                yaml.safe_dump(config, f)
 
-        import jsonargparse_tests
-        setattr(jsonargparse_tests, 'D', D)
-        setattr(jsonargparse_tests, 'A', A)
-        setattr(jsonargparse_tests, 'BSuper', BSuper)
-        setattr(jsonargparse_tests, 'BSub', BSub)
-        setattr(jsonargparse_tests, 'C', C)
+            parser = ArgumentParser()
+            parser.add_argument("--config", action=ActionConfigFile)
+            parser.add_subclass_arguments(BSuper, nested_key="b", required=True)
+            parser.add_class_arguments(C, nested_key="c")
+            parser.link_arguments("c", "b.init_args.a.init_args.d", compute_fn=C.fn, apply_on="instantiate")
 
-        parser = ArgumentParser()
-        parser.add_argument("--config", action=ActionConfigFile)
-        parser.add_subclass_arguments(BSuper, nested_key="b", required=True)
-        parser.add_class_arguments(C, nested_key="c")
-        parser.link_arguments("c", "b.init_args.a.init_args.d", compute_fn=C.fn, apply_on="instantiate")
-
-        config = parser.parse_args(["--config", config_path])
-        config_init = parser.instantiate_classes(config)
-        self.assertIsInstance(config_init["b"].a.d, D)
+            config = parser.parse_args(["--config", config_path])
+            config_init = parser.instantiate_classes(config)
+            self.assertIsInstance(config_init["b"].a.d, D)
 
 
     def test_mapping_class_typehint(self):
@@ -621,33 +625,30 @@ class TypeHintsTmpdirTests(TempDirTestCase):
                 self.class_map = class_map
                 self.int_list = int_list
 
-        import jsonargparse_tests
-        setattr(jsonargparse_tests, 'A', A)
-        setattr(jsonargparse_tests, 'B', B)
+        with mock_module(A, B) as module:
+            parser = ArgumentParser(error_handler=None)
+            parser.add_class_arguments(B, 'b')
 
-        parser = ArgumentParser(error_handler=None)
-        parser.add_class_arguments(B, 'b')
-
-        config = {
-            'b': {
-                'class_map': {
-                    'one': {'class_path': 'jsonargparse_tests.A'},
+            config = {
+                'b': {
+                    'class_map': {
+                        'one': {'class_path': f'{module}.A'},
+                    },
+                    'int_list': [1],
                 },
-                'int_list': [1],
-            },
-        }
+            }
 
-        cfg = parser.parse_object(config)
-        self.assertEqual(cfg.b.class_map, {'one': Namespace(class_path='jsonargparse_tests.A')})
-        self.assertEqual(cfg.b.int_list, [1])
+            cfg = parser.parse_object(config)
+            self.assertEqual(cfg.b.class_map, {'one': Namespace(class_path=f'{module}.A')})
+            self.assertEqual(cfg.b.int_list, [1])
 
-        cfg_init = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg_init.b, B)
-        self.assertIsInstance(cfg_init.b.class_map, dict)
-        self.assertIsInstance(cfg_init.b.class_map['one'], A)
+            cfg_init = parser.instantiate_classes(cfg)
+            self.assertIsInstance(cfg_init.b, B)
+            self.assertIsInstance(cfg_init.b.class_map, dict)
+            self.assertIsInstance(cfg_init.b.class_map['one'], A)
 
-        config['b']['int_list'] = config['b']['class_map']
-        self.assertRaises(ParserError, lambda: parser.parse_object(config))
+            config['b']['int_list'] = config['b']['class_map']
+            self.assertRaises(ParserError, lambda: parser.parse_object(config))
 
 
     def test_linking_deep_targets_mapping(self):
@@ -669,42 +670,66 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             def fn(self) -> D:
                 return D()
 
-        config = {
-            "b": {
-                "class_path": "jsonargparse_tests.BSub",
-                "init_args": {
-                    "a_map": {
-                        "name": {
-                            "class_path": "jsonargparse_tests.A",
+        with mock_module(D, A, BSuper, BSub, C) as module:
+            config = {
+                "b": {
+                    "class_path": f"{module}.BSub",
+                    "init_args": {
+                        "a_map": {
+                            "name": {
+                                "class_path": f"{module}.A",
+                            },
                         },
                     },
                 },
-            },
-            "c": {},
-        }
-        config_path = os.path.join(self.tmpdir, 'config.yaml')
-        with open(config_path, 'w') as f:
-            yaml.safe_dump(config, f)
+                "c": {},
+            }
+            config_path = os.path.join(self.tmpdir, 'config.yaml')
+            with open(config_path, 'w') as f:
+                yaml.safe_dump(config, f)
 
-        import jsonargparse_tests
-        setattr(jsonargparse_tests, 'D', D)
-        setattr(jsonargparse_tests, 'A', A)
-        setattr(jsonargparse_tests, 'BSuper', BSuper)
-        setattr(jsonargparse_tests, 'BSub', BSub)
-        setattr(jsonargparse_tests, 'C', C)
+            parser = ArgumentParser()
+            parser.add_argument("--config", action=ActionConfigFile)
+            parser.add_subclass_arguments(BSuper, nested_key="b", required=True)
+            parser.add_class_arguments(C, nested_key="c")
+            parser.link_arguments("c", "b.init_args.a_map.name.init_args.d", compute_fn=C.fn, apply_on="instantiate")
 
-        parser = ArgumentParser()
-        parser.add_argument("--config", action=ActionConfigFile)
-        parser.add_subclass_arguments(BSuper, nested_key="b", required=True)
-        parser.add_class_arguments(C, nested_key="c")
-        parser.link_arguments("c", "b.init_args.a_map.name.init_args.d", compute_fn=C.fn, apply_on="instantiate")
+            config = parser.parse_args(["--config", config_path])
+            config_init = parser.instantiate_classes(config)
+            self.assertIsInstance(config_init["b"].a_map["name"].d, D)
 
-        config = parser.parse_args(["--config", config_path])
-        config_init = parser.instantiate_classes(config)
-        self.assertIsInstance(config_init["b"].a_map["name"].d, D)
+            config_init = parser.instantiate_classes(config)
+            self.assertIsInstance(config_init["b"].a_map["name"].d, D)
 
-        config_init = parser.instantiate_classes(config)
-        self.assertIsInstance(config_init["b"].a_map["name"].d, D)
+
+    def test_subcommand_with_subclass_default_override_lightning_issue_10859(self):
+
+        class Arch:
+            def __init__(self, a: int = 1):
+                pass
+
+        class ArchB(Arch):
+            def __init__(self, a: int = 2, b: int = 3):
+                pass
+
+        class ArchC(Arch):
+            def __init__(self, a: int = 4, c: int = 5):
+                pass
+
+        parser = ArgumentParser(error_handler=None)
+        parser_subcommands = parser.add_subcommands()
+        subparser = ArgumentParser()
+        subparser.add_argument('--arch', type=Arch)
+
+        with mock_module(Arch, ArchB, ArchC) as module:
+            default = {'class_path': f'{module}.ArchB'}
+            value = {'class_path': f'{module}.ArchC', 'init_args': {'a': 10, 'c': 11}}
+
+            subparser.set_defaults(arch=default)
+            parser_subcommands.add_subcommand('fit', subparser)
+
+            cfg = parser.parse_args(['fit', f'--arch={json.dumps(value)}'])
+            self.assertEqual(cfg.fit.arch.as_dict(), value)
 
 
 class OtherTests(unittest.TestCase):
